==> tty-client-L0.07.tar.gz: tty-client-L0.07/cmd-write.c <==
/*
 *  client/functions-write.c
 *		The functions to actually execute the commands
 *		for the client.  Commands for writing texts.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */



#include <stdio.h>
#include <time.h>
#include <string.h>

#include <zmalloc.h>
#include <s-string.h>
#include <s-collat-tabs.h>

#include <config.h>
#include "config.h"

#include <kom-errno.h>
#include <kom-types.h>
#include <services.h>

#include "copy.h"
#include "xoutput.h"
#include "read-line.h"
#include "commands.h"
#include "parser.h"
#include "texts.h"
#include "edit.h"
#include "error.h"
#include "cache.h"
#include "quit.h"

#include "internal.h"
#include "cmds.h"

#include "defaults.h"

/* This is to emphasize that a function is exported. */
#define Export




/*  Write a new text with the current conference as recipient  */
Export  Success
cmd_write_new_text (String   argument)

{
#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    /*  Create a new text-struct and find a slot for it in the table  */
    active_text = edit_create_new_text();
    /* BUG! Check for -1 in return */
    edit_add_recipient (active_text, ccn);
    /* BUG! Check for FAILURE! */

    newline();
    xprintf ("%s: ", txts.Subject);

    RETURN (do_continue_writing (active_text));

#undef CLEAN_UP
}   /* END: write_new_text() */



/*  Write a comment to a text  */
Export  Success
cmd_comment_text (String   argument)
{
    Text_no		text_to_comment = (Text_no) 0;

    String_size		error_check;
    Text_stat	        text_stat = EMPTY_TEXT_STAT;
    String		text = EMPTY_STRING_i;
    String		subject = EMPTY_STRING_i;

#define CLEAN_UP()	do { s_clear (&argument);\
			     release_text_stat(&text_stat);\
			     s_clear (&text); \
			     s_clear (&subject); } while (0)

    TOPLOOP_SETJMP();

    if (s_empty(argument)) 
    {
	text_to_comment = reading_state.last_viewed;
    } else {
	argument = s_strip_trailing (argument, command_separators);
    
	/* This should be a number, and nothing else. */
	text_to_comment = (Text_no) s_strtol (argument, &error_check,
					      DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
        {
	    /* Wasn't a number. This is an error. */
	    xprintf ("%s: %S\n", txts.Unknown_argument,argument);
	    RETURN (FAILURE);
	}
    }

    if (text_to_comment &&
	kom_get_text_stat (text_to_comment,
			   &text_stat) != FAILURE &&
	kom_get_text (text_to_comment,
		      0, LONGEST_SUBJECT_NOT_TRUNCATED, &text) != FAILURE
	/* BUG: No error-check. */
	)
    {
	Misc_info     * misc;
	int		r;

	active_text = edit_create_new_text();

	for(misc = text_stat.misc_items , r = 0;
	    r < text_stat.no_of_misc;
	    r++, misc++) 
	{
	    if (misc->type == recpt) 
	    {
		edit_add_recipient (active_text, misc->datum.recipient);
	    }
	}
	edit_add_commenting (active_text, text_to_comment);

	if ((r = s_strchr(text, '\n', 0)) == -1) 
	{
	    r = END_OF_STRING;
	}
	s_substr(&subject,text, 0, r);
	edit_add_string(active_text, subject);
    } else {
	xprintf ("%s.\n", txts.Dont_know_what_text_to_comment);
	RETURN (FAILURE);
    }
	
    newline();
    xprintf ("%s:  %S", txts.Subject, *edit_text(active_text));

    RETURN (do_continue_writing (active_text));
#undef CLEAN_UP
}




/*  Reply directly (and privately) to the author of a text  */
Export  Success
cmd_private_comment (String   argument)
{
    Text_no		text_to_comment = (Text_no) 0;

    String_size		error_check;
    Text_stat	        text_stat = EMPTY_TEXT_STAT;
    String		text = EMPTY_STRING_i;
    String		subject = EMPTY_STRING_i;

#define CLEAN_UP()	do { s_clear (&argument);\
			     release_text_stat(&text_stat);\
			     s_clear (&text); \
			     s_clear (&subject); } while (0)

    TOPLOOP_SETJMP();

    if (s_empty(argument)) 
    {
	text_to_comment = reading_state.last_viewed;
    } else {
	argument = s_strip_trailing (argument, command_separators);
    
	/* This should be a number, and nothing else. */
	text_to_comment = (Text_no) s_strtol (argument, &error_check,
					      DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
        {
	    /* Wasn't a number. This is an error. */
	    xprintf ("%s: %S\n", txts.Unknown_argument,argument);
	    RETURN (FAILURE);
	}
    }

    if (text_to_comment &&
	kom_get_text_stat (text_to_comment,
			   &text_stat) != FAILURE &&
	kom_get_text (text_to_comment,
		      0, LONGEST_SUBJECT_NOT_TRUNCATED, &text) != FAILURE
	/* BUG: No error-check. */
	)
    {
	int		r;

	active_text = edit_create_new_text();

	edit_add_recipient (active_text, text_stat.author);
	edit_add_recipient (active_text, cpn);
	edit_add_commenting (active_text, text_to_comment);

	if ((r = s_strchr(text, '\n', 0)) == -1) 
	{
	    r = END_OF_STRING;
	}
	s_substr(&subject,text, 0, r);
	edit_add_string(active_text, subject);
    } else {
	xprintf ("%s.\n", txts.Dont_know_what_text_to_comment);
	RETURN (FAILURE);
    }
	
    newline();
    xprintf ("%s:  %S", txts.Subject, *edit_text(active_text));

    RETURN (do_continue_writing (active_text));
#undef CLEAN_UP
}




/*  Write a letter to a person (or a conference)  */
Export  Success
cmd_write_letter (String   recipient_name)
{
    Micro_conf		  recipient_conf;

#define CLEAN_UP()	do {				\
	s_clear (&recipient_name); } while (0)
    
    TOPLOOP_SETJMP();

    /* BUG:  Should be able to specify more than one recipient */
    recipient_conf = ask_for_conf (&recipient_name, 
				   txts.Who_to_send_letter_to);

    if (recipient_conf.conf_no == 0)
    {
	RETURN (FAILURE);
    }
    active_text = edit_create_new_text();

    edit_add_recipient (active_text, recipient_conf.conf_no);
    edit_add_recipient (active_text, cpn);
    
    newline();
    xprintf ("%s: ", txts.Subject);

    RETURN (do_continue_writing (active_text));

#undef CLEAN_UP
}



/*
 *  Remove the current edited text, i e the user didn't really
 *  want to write the text.
 */
Export  Success
cmd_remove_text (String	argument)

{

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text < 0)
    {
	xprintf ("%s.\n", txts.Youre_not_editing_any_text);
	RETURN (FAILURE);
    }

    active_text = edit_dealloc_text (active_text);

    RETURN (OK);

#undef  CLEAN_UP
}



Export  Success
cmd_enter_text (String	argument)

{
#define FUNCTION "enter_text()"

    Text_no		  number_of_new_text;


#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)


    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    number_of_new_text =
	kom_create_text (edited_texts [active_text] .text,
			 edited_texts [active_text] .no_of_miscs,
			 edited_texts [active_text] .miscs );
    if (number_of_new_text == 0)
    {
	xprintf ("\n%s\n", txts.Text_could_not_be_entered);
	/* BUG! More error codes must be checked for */
	kom_perror ();	/* BUG! Remove this */
	switch (kom_errno)
	{
	case  KOM_NO_CONNECT:
	    fatal (CLIENT_SERVER_ERROR,
				"create_text() failed",
				0);
	    break;

	default:
	    fatal (CLIENT_UNEXPECTED_SERVER_ERROR,
				"create_text() failed",
				0);
	    break;
	}

	RETURN (FAILURE);
    }

    xprintf ("%s %d %s.\n",
	    txts.Text,
	    number_of_new_text,
	    txts.entered_into_db);
    active_text = edit_dealloc_text (active_text);

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}



/* Continue writing a text */
Export  Success
cmd_continue_writing (String	  argument)

{
    int			  pos, start_of_line;
    String		  temp;
    Success		  retval;

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    newline();

    /* Print the line before the insertion point */
    pos = edit_insertion_point (active_text);
    temp = *edit_text (active_text);
    start_of_line = s_strrchr (temp, '\n', pos - 1);
    if (start_of_line < 0)
    {
	xprintf ("%s: ", txts.Subject);
	start_of_line = -1;	/* Not really necessary */
    }
    s_xputs (s_fsubstr (temp, start_of_line + 1, pos - 1));

    retval = do_continue_writing (active_text);

    RETURN (retval);
#undef CLEAN_UP
}


/* Shows the header and the text body sofar. */
Export  Success
cmd_look_at_text_and_header(String argument)
{
    Edited_text		  text;

#define CLEAN_UP()	do { s_clear (&argument); } while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text >= 0) { 
	newline();
	text = edited_texts[active_text];
	print_misc_list( text.miscs, text.no_of_miscs, PRINT_ALL_MISC );
	xprintf ("%s: %S", txts.Subject, *edit_text (active_text));
    }
    else
    {
	cmd_not_yet_implemented (EMPTY_STRING);
	RETURN (FAILURE);
    }

    RETURN (OK);
#undef CLEAN_UP
}
    



/*  ==================================================================	*/
/*			    Internal functions				*/


/*
 *  Continue the writing of a text that was started earlier.
 */
Export  Success
do_continue_writing (int  edit_no)

{
    Edit_result		  edit_return;


#define CLEAN_UP()	do {	} while (0)

    TOPLOOP_SETJMP();

    /* BUG!  Check if any text is active */
    edit_return = edit_enter_text (edit_no);

    switch (edit_return)
    {
    case  enter_this_text:
	cmd_enter_text (EMPTY_STRING);
	break;

    case  kill_this_text:
	active_text = edit_dealloc_text (edit_no);
	break;

    case  to_be_continued:
	break;
    }

    RETURN (OK);

#undef CLEAN_UP
}

==> tty-client-L0.08.tar.gz: tty-client-L0.08/cmd-write.c <==
/*
 *  client/functions-write.c
 *		The functions to actually execute the commands
 *		for the client.  Commands for writing texts.
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */



#include <stdio.h>
#include <time.h>
#include <string.h>

#include <zmalloc.h>
#include <s-string.h>
#include <s-collat-tabs.h>

#include <config.h>
#include "config.h"

#include <kom-errno.h>
#include <kom-types.h>
#include <services.h>

#include "copy.h"
#include "xoutput.h"
#include "read-line.h"
#include "commands.h"
#include "parser.h"
#include "texts.h"
#include "edit.h"
#include "error.h"
#include "cache.h"
#include "quit.h"

#include "internal.h"
#include "cmds.h"

#include "defaults.h"

/* This is to emphasize that a function is exported. */
#define Export




/*  Write a new text with the current conference as recipient  */
Export  Success
cmd_write_new_text (String   argument)

{
#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    /*  Create a new text-struct and find a slot for it in the table  */
    active_text = edit_create_new_text();
    /* BUG! Check for -1 in return */
    edit_add_recipient (active_text, ccn);
    /* BUG! Check for FAILURE! */

    newline();
    xprintf ("%s: ", txts.Subject);

    RETURN (do_continue_writing (active_text));

#undef CLEAN_UP
}   /* END: write_new_text() */



/*  Write a comment to a text  */
Export  Success
cmd_comment_text (String   argument)
{
    Text_no		text_to_comment = (Text_no) 0;

    String_size		error_check;
    Text_stat	        text_stat = EMPTY_TEXT_STAT;
    String		text = EMPTY_STRING_i;
    String		subject = EMPTY_STRING_i;

#define CLEAN_UP()	do { s_clear (&argument);\
			     release_text_stat(&text_stat);\
			     s_clear (&text); \
			     s_clear (&subject); } while (0)

    TOPLOOP_SETJMP();

    if (s_empty(argument)) 
    {
	text_to_comment = reading_state.last_viewed;
    } else {
	argument = s_strip_trailing (argument, command_separators);
    
	/* This should be a number, and nothing else. */
	text_to_comment = (Text_no) s_strtol (argument, &error_check,
					      DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
        {
	    /* Wasn't a number. This is an error. */
	    xprintf ("%s: %S\n", txts.Unknown_argument,argument);
	    RETURN (FAILURE);
	}
    }

    if (text_to_comment &&
	kom_get_text_stat (text_to_comment,
			   &text_stat) != FAILURE &&
	kom_get_text (text_to_comment,
		      0, LONGEST_SUBJECT_NOT_TRUNCATED, &text) != FAILURE
	/* BUG: No error-check. */
	)
    {
	Misc_info     * misc;
	int		r;

	active_text = edit_create_new_text();

	for(misc = text_stat.misc_items , r = 0;
	    r < text_stat.no_of_misc;
	    r++, misc++) 
	{
	    if (misc->type == recpt) 
	    {
		edit_add_recipient (active_text, misc->datum.recipient);
	    }
	}
	edit_add_commenting (active_text, text_to_comment);

	if ((r = s_strchr(text, '\n', 0)) == -1) 
	{
	    r = END_OF_STRING;
	}
	s_substr(&subject,text, 0, r);
	edit_add_string(active_text, subject);
    } else {
	xprintf ("%s.\n", txts.Dont_know_what_text_to_comment);
	RETURN (FAILURE);
    }
	
    newline();
    xprintf ("%s:  %S", txts.Subject, *edit_text(active_text));

    RETURN (do_continue_writing (active_text));
#undef CLEAN_UP
}




/*  Reply directly (and privately) to the author of a text  */
Export  Success
cmd_private_comment (String   argument)
{
    Text_no		text_to_comment = (Text_no) 0;

    String_size		error_check;
    Text_stat	        text_stat = EMPTY_TEXT_STAT;
    String		text = EMPTY_STRING_i;
    String		subject = EMPTY_STRING_i;

#define CLEAN_UP()	do { s_clear (&argument);\
			     release_text_stat(&text_stat);\
			     s_clear (&text); \
			     s_clear (&subject); } while (0)

    TOPLOOP_SETJMP();

    if (s_empty(argument)) 
    {
	text_to_comment = reading_state.last_viewed;
    } else {
	argument = s_strip_trailing (argument, command_separators);
    
	/* This should be a number, and nothing else. */
	text_to_comment = (Text_no) s_strtol (argument, &error_check,
					      DEFAULT_NUMBER_BASE);
	if (error_check < s_strlen (argument))
        {
	    /* Wasn't a number. This is an error. */
	    xprintf ("%s: %S\n", txts.Unknown_argument,argument);
	    RETURN (FAILURE);
	}
    }

    if (text_to_comment &&
	kom_get_text_stat (text_to_comment,
			   &text_stat) != FAILURE &&
	kom_get_text (text_to_comment,
		      0, LONGEST_SUBJECT_NOT_TRUNCATED, &text) != FAILURE
	/* BUG: No error-check. */
	)
    {
	int		r;

	active_text = edit_create_new_text();

	edit_add_recipient (active_text, text_stat.author);
	edit_add_recipient (active_text, cpn);
	edit_add_commenting (active_text, text_to_comment);

	if ((r = s_strchr(text, '\n', 0)) == -1) 
	{
	    r = END_OF_STRING;
	}
	s_substr(&subject,text, 0, r);
	edit_add_string(active_text, subject);
    } else {
	xprintf ("%s.\n", txts.Dont_know_what_text_to_comment);
	RETURN (FAILURE);
    }
	
    newline();
    xprintf ("%s:  %S", txts.Subject, *edit_text(active_text));

    RETURN (do_continue_writing (active_text));
#undef CLEAN_UP
}




/*  Write a letter to a person (or a conference)  */
Export  Success
cmd_write_letter (String   recipient_name)
{
    Micro_conf		  recipient_conf;

#define CLEAN_UP()	do {				\
	s_clear (&recipient_name); } while (0)
    
    TOPLOOP_SETJMP();

    /* BUG:  Should be able to specify more than one recipient */
    recipient_conf = ask_for_conf (&recipient_name, 
				   txts.Who_to_send_letter_to);

    if (recipient_conf.conf_no == 0)
    {
	RETURN (FAILURE);
    }
    active_text = edit_create_new_text();

    edit_add_recipient (active_text, recipient_conf.conf_no);
    edit_add_recipient (active_text, cpn);
    
    newline();
    xprintf ("%s: ", txts.Subject);

    RETURN (do_continue_writing (active_text));

#undef CLEAN_UP
}



/*
 *  Remove the current edited text, i e the user didn't really
 *  want to write the text.
 */
Export  Success
cmd_remove_text (String	argument)

{

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text < 0)
    {
	xprintf ("%s.\n", txts.Youre_not_editing_any_text);
	RETURN (FAILURE);
    }

    active_text = edit_dealloc_text (active_text);

    RETURN (OK);

#undef  CLEAN_UP
}



Export  Success
cmd_enter_text (String	argument)

{
#define FUNCTION "enter_text()"

    Text_no		  number_of_new_text;


#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)


    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text < 0)
    {
	xprintf ("%s.\n", txts.Youre_not_editing_any_text);
	RETURN (FAILURE);
    }

    number_of_new_text =
	kom_create_text (edited_texts [active_text] .text,
			 edited_texts [active_text] .no_of_miscs,
			 edited_texts [active_text] .miscs );
    if (number_of_new_text == 0)
    {
	xprintf ("\n%s\n", txts.Text_could_not_be_entered);
	/* BUG! More error codes must be checked for */
	kom_perror ();	/* BUG! Remove this */
	switch (kom_errno)
	{
	case  KOM_NO_CONNECT:
	    fatal (CLIENT_SERVER_ERROR,
				"create_text() failed",
				0);
	    break;

	default:
	    fatal (CLIENT_UNEXPECTED_SERVER_ERROR,
				"create_text() failed",
				0);
	    break;
	}

	RETURN (FAILURE);
    }

    xprintf ("\n%s %d %s.\n",
	    txts.Text,
	    number_of_new_text,
	    txts.entered_into_db);
    active_text = edit_dealloc_text (active_text);

    RETURN (OK);

#undef CLEAN_UP
#undef FUNCTION
}



/* Continue writing a text */
Export  Success
cmd_continue_writing (String	  argument)

{
    int			  pos, start_of_line;
    String		  temp;
    Success		  retval;

#define CLEAN_UP()	do {	s_clear (&argument);	} while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text < 0)
    {
	xprintf ("%s.\n", txts.Youre_not_editing_any_text);
	RETURN (FAILURE);
    }
    newline();

    /* Print the line before the insertion point */
    pos = edit_insertion_point (active_text);
    temp = *edit_text (active_text);
    start_of_line = s_strrchr (temp, '\n', pos - 1);
    if (start_of_line < 0)
    {
	xprintf ("%s: ", txts.Subject);
	start_of_line = -1;	/* Not really necessary */
    }
    s_xputs (s_fsubstr (temp, start_of_line + 1, pos - 1));

    retval = do_continue_writing (active_text);

    RETURN (retval);
#undef CLEAN_UP
}


/* Shows the header and the text body sofar. */
Export  Success
cmd_look_at_text_and_header(String argument)
{
    Edited_text		  text;

#define CLEAN_UP()	do { s_clear (&argument); } while (0)

    TOPLOOP_SETJMP();

    CHECK_NO_ARGUMENTS();

    if (active_text >= 0) {	/*  */
	newline();
	text = edited_texts[active_text];
	print_misc_list( text.miscs, text.no_of_miscs, PRINT_ALL_MISC );
	xprintf ("%s: %S", txts.Subject, *edit_text (active_text));
    }
    else
    {
	xprintf ("%s\n", txts.Youre_not_editing_any_text);
	RETURN (FAILURE);
    }

    RETURN (OK);
#undef CLEAN_UP
}
    



/*  ==================================================================	*/
/*			    Internal functions				*/


/*
 *  Continue the writing of a text that was started earlier.
 */
Export  Success
do_continue_writing (int  edit_no)

{
    Edit_result		  edit_return;


#define CLEAN_UP()	do {	} while (0)

    TOPLOOP_SETJMP();

    /* BUG!  Check if any text is active */
    edit_return = edit_enter_text (edit_no);

    switch (edit_return)
    {
    case  enter_this_text:
	cmd_enter_text (EMPTY_STRING);
	break;

    case  kill_this_text:
	active_text = edit_dealloc_text (edit_no);
	break;

    case  to_be_continued:
	break;
    }

    RETURN (OK);

#undef CLEAN_UP
}
